__title__ = 'soyspacing'
__version__ = '0.1.21'
__author__ = 'Lovit'
__license__ = 'GPL v3'
__copyright__ = 'Copyright 2017 Lovit'

from . import countbase
from . import crusuite
from .hangle import normalize
from .hangle import normalize_text